---
id: ecosystem
title:  "Ecosystem"
---

### If you find a new great library, talk, resource, slides or project, related to ZIO, consider adding to the list with your PR

## Official ZIO Libraries

These libraries are hosted in the [ZIO organization](https://github.com/zio/) on Github, and are generally maintained by core contributors to ZIO.

- [ZIO Actors](https://github.com/zio/zio-actors): A high-performance, purely-functional library for building, composing, and supervising typed actors based on ZIO
- [ZIO Akka Cluster](https://github.com/zio/zio-akka-cluster): A ZIO wrapper for Akka Cluster
- [ZIO Config](https://github.com/zio/zio-config): A ZIO based configuration parsing library
- [ZIO Web](https://github.com/zio/zio-web): A ZIO-powered HTTP server and client
- [ZIO Kafka](https://github.com/zio/zio-kafka): A Kafka client for ZIO and ZIO Streams
- [ZIO Keeper](https://github.com/zio/zio-keeper): A functional library for consistent replication of metadata across dynamic clusters
- [ZIO Logging](https://github.com/zio/zio-logging): An environmental effect for adding logging into any ZIO application, with choice of pluggable back-ends
- [ZIO NIO](https://github.com/zio/zio-nio): A performant, purely-functional, low-level, and unopinionated wrapper around Java NIO functionality
- [ZIO SQS](https://github.com/zio/zio-sqs): A ZIO-powered client for AWS SQS
- [ZIO Telemetry](https://github.com/zio/zio-telemetry): A ZIO-powered OpenTelemetry library 


## Libraries with Direct ZIO Support

If you know a useful library that has direct support for ZIO, please consider [submitting a pull request](https://github.com/zio/zio/pulls) to add it to this list.

- [cakeless](https://github.com/itkpi/cakeless): Wire your cakes automatically into zio environment
- [caliban](https://github.com/ghostdogpr/caliban): Functional GraphQL backend in Scala
- [d4s](https://github.com/PlayQ/d4s): "Dynamo DB Database done Scala way". A library that allows accessing the DynamoDB in a purely functional way
- [distage](https://github.com/7mind/izumi): Staged, transparent and debuggable runtime & compile-time Dependency Injection Framework
- [elastic4s](https://github.com/sksamuel/elastic4s): Elasticsearch Scala Client - Reactive, Non Blocking, Type Safe, HTTP Client
- [idealingua](https://github.com/7mind/izumi): API Definition, Data Modeling and RPC Language, optimized for fast prototyping – like gRPC, but with a human face
- [logstage](https://github.com/7mind/izumi): Automatic structural logs from Scala string interpolations
- [neotypes](https://github.com/neotypes/neotypes): A Scala lightweight, type-safe & asynchronous driver for neo4j
- [rezilience](https://github.com/svroonland/rezilience): Utilities for resilience and handling of transient errors
- [scanamo](https://github.com/scanamo/scanamo): Simpler DynamoDB access for Scala
- [slf4zio](https://github.com/mlangc/slf4zio): Simple convenience layer on top of SLF4J for ZIO
- [sttp](https://github.com/softwaremill/sttp): The Scala HTTP client you always wanted!
- [tranzactio](https://github.com/gaelrenoux/tranzactio): ZIO wrapper for data access libraries like Doobie or Anorm
- [ZIO Arrow](https://github.com/Neurodyne/zio-arrow) : Haskell Arrow meets ZIO. A deep composition and high performance applications
- [zio-amqp](https://github.com/svroonland/zio-amqp): ZIO Streams based RabbitMQ client
- [zio-aws](https://github.com/vigoo/zio-aws): Low-level AWS wrapper for ZIO for all AWS services using the AWS Java SDK v2
- [zio-aws-s3](https://github.com/Neurodyne/zio-aws-s3): A lean, simple and efficient ZIO wrapper for AWS Java v2 S3 API by Boris V.Kuznetsov
- [zio-easymock](https://github.com/egast/zio-easymock): EasyMock mocking for zio-test
- [zio-email](https://github.com/funcit/zio-email): Purely functional email client
- [zio-event-sourcing](https://github.com/holinov/zio-event-sourcing): Purely functional concurrent and scalable persistance layer
- [zio-grpc](https://github.com/scalapb/zio-grpc): A native gRPC support for ZIO
- [zio-interop-log4j2](https://github.com/mlangc/zio-interop-log4j2): Fiber aware MDC logging for Log4j 2
- [zio-kinesis](https://github.com/svroonland/zio-kinesis): ZIO Streams based AWS Kinesis client
- [zio-saga](https://github.com/VladKopanev/zio-saga): Purely functional transaction management with Saga pattern
- [zio-slf4j](https://github.com/NeQuissimus/zio-slf4j): Referentially transparent logging with slf4j
- [zio-slick](https://github.com/rleibman/zio-slick): Bridge library between ZIO and Slick Functional Relational Mapping Library


## Tools for ZIO

- [ZIO IntelliJ](https://github.com/zio/zio-intellij): A complimentary, community-developed plugin for IntelliJ IDEA, brings enhancements when using ZIO in your projects
- [zio-shield](https://github.com/zio/zio-shield): Enforce best coding practices with ZIO
- [zio-akka-quickstart.g8](https://github.com/ScalaConsultants/zio-akka-quickstart.g8): A Giter8 template for a basic Scala application build using ZIO, Akka HTTP and Slick
- [zio-dotty-quickstart.g8](https://github.com/ScalaConsultants/zio-dotty-quickstart.g8): A Giter8 template for a basic Dotty application build using ZIO


## ZIO Interoperability Libraries

ZIO provides the ability to interoperate with other parts of the broader ecosystem, see [Interop](../interop/index.md) for overview.
